
MixerChannelGUI {
		// stores relationship between MixerChannel and its display views
		// assumes MixingBoard has already been created

	var	<mixer,	// the instance of mixerchannel
		<board,	// the instance of mixingboard
//		<>index,	// index to channel's position on board
		<>origin,	// provided by MixingBoard
		
			// view variables
		<levelSlider, <panSlider, <nameView, <levelView,
		<preSendMenus, <preSendLevels, <postSendMenus, <postSendLevels,
		<preSendOldValues, <postSendOldValues,	// for send menu actions
		<destMenu,	// identify outbus

	 	<>panSpec,	// each mixerchannel can use a different panSpec
					// to map slider to pan value

		<color1, <color2,		// each channel can override skin's color & font settings
		<textFont,			// nil's in any of these vars will use skin value
							// for this variable only
							// you can't override view sizes!
		panSliderBackground, levelSliderBackground;	// store local backgrounds

	*new {
		arg mixer, board, index;
		^super.new.init(mixer, board, index)
	}
	
	init {
		arg mix, bd, ind;
		
			// if index is omitted, fetch it
//		index = ind ? MixingBoard.boards.indexOf(bd);
		mixer = mix;
		board = bd;
		
		panSpec = ControlSpec.new(-1, 1, \linear, 0, 0);

		this.makeViews;

		mixer.mcgui = this;	// save path for mixerchannel to reach these views
		^this
	}
	
	makeViews {
			// simply creates the view objects; does not position them
			// because that depends on some things from MixingBoard
		{ levelSlider = SCSlider.new(board.w, board.skin.levelSlider).step_(0.01)
			.action_(this.levelAction);
		panSlider = SCSlider.new(board.w, board.skin.panSlider).step_(0.01)
			.action_(this.panAction);
		nameView = SCStaticText.new(board.w, board.skin.nameBounds).align_(\center);
		levelView = SCStaticText.new(board.w, board.skin.levelView).align_(\center);
		destMenu = SCPopUpMenu.new(board.w, board.skin.destMenu)
				.action_(this.destMenuAction);
				// make subarrays for sends
		this.addPreSends(board.skin.preSendsShow);
		this.addPostSends(board.skin.postSendsShow);
		nil }.defer;
	}
	
	addPreSends { arg num;
		var oldsize;
		oldsize = preSendMenus.isNil.if({ 0 }, { preSendMenus.size });
			// add new send views to the existing array, or a blank one
		preSendMenus = ((preSendMenus ? Array.new) ++ Array.fill(num, 
			{ arg i;
				SCPopUpMenu.new(board.w, board.skin.firstPreSendDest)
				.action_(this.preSendMenuAction(i + oldsize))
			})).flat;
		preSendLevels = ((preSendLevels ? Array.new) ++ Array.fill(num, 
			{ arg i;
				SCSlider.new(board.w, board.skin.firstPreSendLevel)
				.action_(this.preSendLevelAction(i + oldsize))
			})).flat;
		preSendOldValues = ((preSendOldValues ? Array.new) ++ 
			Array.fill(num, -1)).flat;
	}
	
	addPostSends { arg num;
		var oldsize;
		oldsize = postSendMenus.isNil.if({ 0 }, { postSendMenus.size });
			// add new send views to the existing array, or a blank one
		postSendMenus = ((postSendMenus ? Array.new) ++ Array.fill(num, 
			{ arg i;
				SCPopUpMenu.new(board.w, board.skin.firstPostSendDest)
				.action_(this.postSendMenuAction(i + oldsize))
			})).flat;
		postSendLevels = ((postSendLevels ? Array.new) ++ Array.fill(num, 
			{ arg i;
				SCSlider.new(board.w, board.skin.firstPostSendLevel)
				.action_(this.postSendLevelAction(i + oldsize))
			})).flat;
		postSendOldValues = ((postSendOldValues ? Array.new) ++ 
			Array.fill(num, -1)).flat;
	}
	
	refresh {
			// update everything about these views
		var sendItems, j, send;
		
			// list of buses
		sendItems = BusDict.menuItems(mixer.server) ++ [\none];

		{ levelSlider.bounds = board.skin.levelSlider.moveBy(origin.x, origin.y);
		levelView.bounds = board.skin.levelView.moveBy(origin.x, origin.y);
		panSlider.bounds = board.skin.panSlider.moveBy(origin.x, origin.y);
		nameView.bounds = board.skin.nameBounds.moveBy(origin.x, origin.y);
		destMenu.bounds_(board.skin.destMenu.moveBy(origin.x, origin.y))
			.items_(sendItems).font_(textFont ? board.skin.textFont);
		
			// do we need more send views?
		(preSendMenus.size < board.skin.preSendsShow).if({
			this.addPreSends(board.skin.preSendsShow - preSendMenus.size);
		});
		(postSendMenus.size < board.skin.postSendsShow).if({
			this.addPostSends(board.skin.postSendsShow - postSendMenus.size);
		});
		
			// do we need to push some views away?
			// no good way to remove views, so push them into a little mole at the top left
		(preSendMenus.size > board.skin.preSendsShow).if({
			for(board.skin.preSendsShow, preSendMenus.size, {
				arg i;
				preSendMenus.at(i).bounds = Rect.new;
				preSendLevels.at(i).bounds = Rect.new;
			});
		});

		(postSendMenus.size > board.skin.postSendsShow).if({
			for(board.skin.postSendsShow, postSendMenus.size, {
				arg i;
				postSendMenus.at(i).bounds = Rect.new;
				postSendLevels.at(i).bounds = Rect.new;
			});
		});

			// this should probably be handled in another class, but not yet
		board.skin.preSendsShow.do({ arg j;
			(preSendMenus@j).bounds_(board.skin.firstPreSendDest
				.moveBy(	origin.x + (board.skin.preSendIncr.x * j),
						origin.y + (board.skin.preSendIncr.y * j)));
			(preSendMenus@j).items = sendItems;
			(preSendLevels@j).bounds_(board.skin.firstPreSendLevel
				.moveBy(	origin.x + (board.skin.preSendIncr.x * j),
						origin.y + (board.skin.preSendIncr.y * j)));
				// do these values now to avoid doing this loop again
			send = mixer.preSends@(j /* + first send displayed */);
			send.notNil.if({
				(preSendLevels@j).setProperty(\value, send.level);
				(preSendMenus@j).setProperty(\value, send.outbus.index);
				(preSendMenus@j).font_(textFont ? board.skin.textFont);
				(preSendOldValues).put(j, send.outbus.index);
				send.guiIndex = j;
			}, {
				// if no send exists, leave level alone and say "none" for dest
				(preSendMenus@j).setProperty(\value,
					(preSendMenus@j).items.size-1);
				(preSendMenus@j).font_(textFont ? board.skin.textFont);
				(preSendOldValues).put(j, (preSendMenus@j).items.size-1);
			});
		});
		board.skin.postSendsShow.do({ arg j;
			(postSendMenus@j).bounds_(board.skin.firstPostSendDest
				.moveBy(	origin.x + (board.skin.postSendIncr.x * j),
						origin.y + (board.skin.postSendIncr.y * j)));
			(postSendMenus@j).items = sendItems;
			(postSendLevels@j).bounds_(board.skin.firstPostSendLevel
				.moveBy(	origin.x + (board.skin.postSendIncr.x * j),
						origin.y + (board.skin.postSendIncr.y * j)));
			send = mixer.postSends@(j /* + first send displayed */);
			send.notNil.if({
				(postSendLevels@j).setProperty(\value, send.level);
				(postSendMenus@j).setProperty(\value, send.outbus.index);
				(postSendMenus@j).font_(textFont ? board.skin.textFont);
				(postSendOldValues).put(j, send.outbus.index);
				send.guiIndex = j;
			}, {
				// if no send exists, leave level alone and say "none" for dest
				(postSendMenus@j).setProperty(\value,
					(postSendMenus@j).items.size-1);
				(postSendMenus@j).font_(textFont ? board.skin.textFont);
				(postSendOldValues).put(j, (postSendMenus@j).items.size-1);
			});
		});
		
			// fill in values -- use updater methods below?
		levelSlider.setProperty(\value, mixer.level);
		panSlider.setProperty(\value,
			panSpec.unmap(mixer.pan));
		levelView.string_(levelSlider.value.ampdb.trunc(0.01))
				.font_(textFont ? board.skin.textFont);
		nameView.string_(mixer.name).font_(textFont ? board.skin.textFont);
		destMenu.setProperty(\value, mixer.outbus.index);
		
			// have colors changed?
		levelSlider.background = levelSliderBackground ? board.skin.levelSliderBackground;
		panSlider.background = panSliderBackground ? board.skin.panSliderBackground;
		nil }.defer;
	}
	
			// user should not call this alone! this should be called from MixingBoard-remove
	free { arg freeIt = true;
		freeIt.if({ mixer.free(false); }); 	// free mixer & don't update gui
//		{ board.w.view.children.remove(levelSlider);
//		board.w.view.children.remove(panSlider);
//		board.w.view.children.remove(levelView);
//		board.w.view.children.remove(nameView);
//		board.skin.preSendsShow.do({ arg i;
//			board.w.view.children.remove(preSendMenus.at(i));
//			board.w.view.children.remove(preSendLevels.at(i));
//		});
//		board.skin.postSendsShow.do({ arg i;
//			board.w.view.children.remove(postSendMenus.at(i));
//			board.w.view.children.remove(postSendLevels.at(i));
//		});
//		nil }.defer;
		{ levelSlider.bounds = Rect.new;
		panSlider.bounds = Rect.new;
		levelView.bounds = Rect.new;
		nameView.bounds = Rect.new;
		destMenu.bounds = Rect.new;
		board.skin.preSendsShow.do({ arg i;
			preSendMenus.at(i).bounds = Rect.new;
			preSendLevels.at(i).bounds = Rect.new;
		});
		board.skin.postSendsShow.do({ arg i;
			postSendMenus.at(i).bounds = Rect.new;
			postSendLevels.at(i).bounds = Rect.new;
		});
		nil }.defer;
			// update others? board responsibility?
	}
	
	// update a single view
	
	levelUpdate { arg l;
		{ levelSlider.setProperty(\value, l);
		levelView.string = l.ampdb.trunc(0.01).asString;
		nil }.defer;
	}
	
	panUpdate { arg p;
		{ panSlider.setProperty(\value, panSpec.unmap(p));
		nil }.defer;
	}
	
	nameUpdate { arg n;
		{ nameView.string = n;
		{ board.w.refresh; nil }.defer(0.1);
		nil }.defer;
	}
	
	preSendMenuUpdate { arg i, b;	// index to presend, bus number
		{ preSendMenus.at(i).setProperty(\value, b);
		nil }.defer;
	}
	
	preSendLevelUpdate { arg i, l;
		{ preSendLevels.at(i).setProperty(\value, l);
		nil }.defer;
	}

	postSendMenuUpdate { arg i, b;	// index to postsend, bus number
		{ postSendMenus.at(i).setProperty(\value, b);
		nil }.defer;
	}
	
	postSendLevelUpdate { arg i, l;
		{ postSendLevels.at(i).setProperty(\value, l);
		nil }.defer;
	}
	
	destMenuUpdate { arg busindex;
		{ destMenu.setproperty(\value, busindex);
		nil }.defer;
	}
	
		// setters to override skin settings
	color1_ { arg color, refresh;
		color1 = color;
		this.fixBackgrounds;
		{ (refresh ? board.skin.autoRefresh).if({ MixingBoard.facelift });
		nil }.defer;
		^this
	}
	
	color2_ { arg color, refresh;
		color2 = color;
		this.fixBackgrounds;
		{ (refresh ? board.skin.autoRefresh).if({ MixingBoard.facelift });
		nil }.defer;
		^this
	}

	textFont_ { arg font, refresh;
		textFont = font;
		{ (refresh ? board.skin.autoRefresh).if({ MixingBoard.facelift });
		nil }.defer;
		^this
	}
	
	fixBackgrounds {
		var c1, c2;
		c1 = color1 ? board.skin.color1;
		c2 = color2 ? board.skin.color2;
		panSliderBackground = HiliteGradient(c2, c1, \h, 50, 0.5);				// note, pan is always horiz.
		levelSliderBackground = (board.skin.layout == \horizontal).if(
			{ Gradient(c1, c2, \h, 50) },
			{ Gradient(c2, c1, \v, 50) }
		);
		
		^this
	}

	// action templates
	
	levelAction {
//		arg i;	// index of mixerchannel to operate on
		
		^{
			levelView.string = levelSlider.value.ampdb.trunc(0.01);
				// false = don't redraw gui (since gui is causing change)
				// true = stop automation on user touch
			mixer.level_(levelSlider.value, false, true);
		}
	}
	
	panAction {
//		arg i;	// index of mixerchannel to operate on
		
		^{
			mixer.pan_(panSpec.map(panSlider.value), false, true);
		}
	}
	
	preSendMenuAction {
		arg j;		// index of mixerchannel, presend to operate on
		
		// j = j + first send displayed
		
		^{
				// if value is unchanged, do nothing
			((preSendMenus@j).value != preSendOldValues@j).if({
					// if old value is "none," create a presend
				(preSendOldValues@j == ((preSendMenus@j).items.size-1)).if({
						// default level is 0
	"Making presend".postln;
					MixerPreSend.new(mixer, (preSendMenus@j).value, 0);
					(preSendLevels@j).setProperty(\value, 0);
				}, {
	("Freeing presend " ++ j).postln;
						// if new value is "none," free the presend
					((preSendMenus@j).value == ((preSendMenus@j).items.size-1)).if({
						(mixer.preSends@j).free;
					}, {
	("Repatching send from bus " ++ (mixer.preSends@j).outbus.index ++ " to " ++ 
		(preSendMenus@j).value).postln;
							// otherwise, just change the destination
						(mixer.preSends@j).outbus = (preSendMenus@j).value;
					});
				});
			});
				// save for next call
			(preSendOldValues).put(j, (preSendMenus@j).value);
		}
	}
	
	preSendLevelAction {
		arg j;
		
		// j = j +...
			// set the level, don't update gui (b/c user action did)
		^{ (mixer.preSends@j).level_((preSendLevels@j).value, false); }
	}

	postSendMenuAction {
		arg j;		// index of mixerchannel, postsend to operate on
		
		// j = j + first send displayed
		
		^{
				// if value is unchanged, do nothing
			((postSendMenus@j).value != postSendOldValues@j).if({
					// if old value is "none," create a postsend
				(postSendOldValues@j == ((postSendMenus@j).items.size-1)).if({
						// default level is 0
	"Making postsend".postln;
					MixerPostSend.new(mixer, (postSendMenus@j).value, 0);
					(postSendLevels@j).setProperty(\value, 0);
				}, {
	("Freeing postsend " ++ j).postln;
						// if new value is "none," free the postsend
					((postSendMenus@j).value == ((postSendMenus@j).items.size-1)).if({
						(mixer.postSends@j).free;
					}, {
	("Repatching send from bus " ++ (mixer.postSends@j).outbus.index ++ " to " ++ 
		(postSendMenus@j).value).postln;
							// otherwise, just change the destination
						(mixer.postSends@j).outbus_((postSendMenus@j).value, false);
					});
				});
			});
				// save for next call
			(postSendOldValues).put(j, (postSendMenus@j).value);
		}
	}
	
	postSendLevelAction {
		arg j;
		
		// j = j +...
			// set the level, don't update gui (b/c user action did)
		^{ (mixer.postSends@j).level_((postSendLevels@j).value, false); }
	}
	
	destMenuAction {
		^{ mixer.outbus_(destMenu.value, false); }
	}
}
