MixerSend {
		// container for aux sends for MixerChannels & boards
		// input must be a mixerchannel; output can be an mc, bus or bus#

	var	<inMixer, <outbus, <levelbus, <level,
		<sendSynth, <levelSynth, <guiUpdater = nil,
		<>guiIndex;	// change this from outside at your peril!
		
	var <>guiUpdateTime = 0.25;		// see .levelLine / .levelTo
		// creation & init methods in MixerPre/PostSend
		
		// setting level
	level_ { arg lev, updateGUI = true, stopAutomation = true;
		var slider;	// for gui update
		stopAutomation.if({ this.stopLevelAuto; });
			// gui update
		(updateGUI && inMixer.mcgui.notNil && guiIndex.notNil).if({
			slider = this.isKindOf(MixerPreSend).if(
				{ inMixer.mcgui.preSendLevels.at(guiIndex) },
				{ inMixer.mcgui.postSendLevels.at(guiIndex) }
			);
			slider.notNil.if({  // if board/mixer/send indices are bad, slider == nil
				{ slider.setProperty(\value, lev); nil }.defer;
			});
		});
				
		level = lev;
		sendSynth.set(\level, level);
		^this
	}
	
		// automation
	levelAuto {
			// place a .kr synthdef onto level control bus
		arg synthdef ... args;
		var dur, end;
		
			// if levelSynth is active, kill it
		this.stopLevelAuto;

			// schedule monitoring for gui

		args.includes(\end).if({	// if a finite synthdef,
			dur = args.at(args.indexOf(\dur) + 1);
			end = args.at(args.indexOf(\end) + 1);
			SystemClock.sched(		// schedule update for level inst var
				dur,	{ this.level_(end, true, false) }
			);
		});

			// create levelbus if it doesn't exist
		levelbus.isKindOf(Bus).not.if({
			levelbus = BusDict.control(inMixer.server, 1, "send level");
		});

//("Synth args: " ++ synthdef ++ ", ").post;
		levelSynth = Synth.head(inMixer.fadergroup, "mxaut/" ++ synthdef, 
//(
			args.flat ++ args.includes(\busout).if(
				{ nil }, { [\busout, levelbus.index] })
//).postln;
		);
		sendSynth.map(\level, levelbus.index);
		^levelSynth
	}
	
		// line automation -- ...To uses current value as start
		// any warp that can be used in a ControlSpec can be used here
	levelLine {
		arg start = 0, end = 1, dur = 1, warp = \linear;
		
		var startTime, timeSpec;
		this.stopLevelAuto;	// begin by clearing automation

			// start the automation process on the server
			// this line will be revived when bus.get is implemented
//		this.levelAuto("krLine", \start, start, \end, end, \dur, dur);
		
			// schedule periodic updates for level and gui
			// spec to convert time into level
//"creating timeSpec".postln;
		timeSpec = ControlSpec.new(start, end, warp);
		startTime = Main.elapsedTime;
//"starting gui updater routine".postln;
		guiUpdater = Routine({
			{ (Main.elapsedTime - startTime) <= dur }.while({
				this.level_(timeSpec.map(
					(Main.elapsedTime-startTime) / dur
				), true, false);  // update gui but don't stop automation
				guiUpdateTime.yield	// wait before going again
			});
//"stopping routine".postln;
			nil.yield		// stop
		}).play(SystemClock);
	}
	
	levelTo {		// start from current level
		arg end, dur, warp = \linear;
		
		this.levelLine(level, end, dur, warp);
	}
	
	stopLevelAuto {
		levelSynth.notNil.if({
			sendSynth.map(\level, -1);	// return to static level
			levelSynth.free;			// kill kr synth
			BusDict.free(levelbus);
			levelSynth = nil;
		});
		guiUpdater.isNil.not.if({
			guiUpdater.stop;
			guiUpdater = nil;
		});
		^this
	}
	
	outbus_ {		// repatch send to another bus
		arg bus, updateGUI = true, target, moveAction;
		var mc; 
		
		bus.isKindOf(MixerChannel).if({
			bus = bus.inbus;
		});
		bus.isKindOf(Integer).if({
			bus = Bus.new(\audio, bus, inMixer.inChannels, inMixer.server);
		});
		
			// if the new bus has an entry in the MixerChannel table
		(mc = MixerChannel.servers.at(inMixer.server).at(bus.index)).notNil.if({
				// if the dest mixer is routed to another mixer,
				// place the dest mixer before its destination
			(moveAction.isNil && mc.outbus.asMixer.notNil).if({
				moveAction = \moveBefore;
				target = mc.outbus.asMixer.fadergroup;
			}, {		// otherwise, if target not specified, move to tail of whole server
//"send.outbus_: fixing order of execution".postln;
				moveAction = \moveToTail;
				target = target ? mc.server.asGroup;
			});
			mc.fadergroup.perform(moveAction, target);
		});

		sendSynth.set(\busout, bus.index);
		outbus = bus;
		updateGUI.if({ inMixer.mcgui.refresh });
	}

}	

MixerPreSend : MixerSend {

	*new { arg in, out, level = 1;
		^super.new.initPreSend(in, out, level);
	}

	initPreSend { arg in1, out1, lev = 1, target, moveAction;
		
		inMixer = in1;	// must be mixerchannel in
		level = lev;
		
			// Buses should be left alone, other types are fixed by following rules:
		out1.isKindOf(Bus).if({
			outbus = out1;
		}, {
				// support for mixerchannels in
			out1.isKindOf(MixerChannel).if({
				outbus = out1.inbus;
			});
			
				// support for sending bus # for out
			out1.isKindOf(Integer).if({
				outbus = Bus.new(\audio, out1, inMixer.inChannels, inMixer.server);
			});
			
			outbus.isNil.if({		// create outbus if we still don't have it
				outbus = BusDict.audio(inMixer.server, inMixer.inChannels,
					inMixer.name ++ " presend");
			});
		});

//inMixer.inbus.dump; outbus.dump;
		
			// check numchannels
		(inMixer.inChannels != outbus.numChannels).if({
			("Warning: numChannel mismatch creating send for " ++ inMixer.name
				++ " -- " ++ inMixer.outChannels ++ " --> " ++ outbus.numChannels).postln;
		});
		
		level = lev;

		sendSynth = Synth.before(inMixer.synth, // place before fader unit
			"mixers/Send" ++ inMixer.inChannels,
			[\busin, inMixer.inbus.index, \busout, outbus.index,
			 \level, level]
		);
		
		guiIndex = inMixer.preSends.size;
		inMixer.preSends = inMixer.preSends.add(this);
			// fix gui display
		inMixer.mcgui.notNil.if({ inMixer.mcgui.refresh });
		
		this.outbus_(outbus, false, target, moveAction);   // fix order of execution
		
		^this
	}
	
	free {
		arg updateGUI = true;
		
//		updateGUI.if({
//		});

		this.stopLevelAuto;	// cancel automation

		inMixer.preSends.remove(this);

		sendSynth.free;	// stop sending
		BusDict.free(levelbus);		// free the kr bus

			// fix gui display
		(inMixer.mcgui.notNil && updateGUI).if({ inMixer.mcgui.board.refresh });

	}
	
}

MixerPostSend : MixerSend {
		// additional inst variables needed for routing.
		// a post-send creates a bus (sendbus) which receives
		// the output of the input mixerchannel. then synth
		// objects transfer the input to the input mixerchannel's
		// output and to the send destination
		// these are also reflected in the MixerChannel instance
		// (to avoid creating redundant buses and transfer synths)

	var sendbus, xferSynth;
	
	*new { arg in, out, level = 1;
		^super.new.initPostSend(in, out, level);
	}

	initPostSend { arg in1, out1, lev = 1, target, moveAction;
		
		inMixer = in1;	// must be mixerchannel in
		
		level = lev;
		
		out1.isKindOf(Bus).if({
			outbus = out1;
		}, {
				// support for mixerchannels for out
			out1.isKindOf(MixerChannel).if({
				outbus = out1.inbus;
			});
			
				// support for bus # for out
			out1.isKindOf(Integer).if({
				outbus = BusDict.audioObjects.at(inMixer.server).at(out1);
			});
			
			outbus.isNil.if({		// create outbus if we still don't have it
				outbus = BusDict.audio(inMixer.server, inMixer.inChannels,
					inMixer.name ++ " postsend");
			});
		});
//outbus.dump;
		
			// if inMixer uses the post-send ready architecture, we don't need
			// to make a transfer synth and bus
		inMixer.postSendReady.not.if({
			inMixer.postSendBus.isNil.if({
				sendbus = BusDict.audio(inMixer.server, inMixer.outChannels,
					"send xfer bus");
				inMixer.postSendBus = sendbus;	// retain in mixerchannel
			}, {
				sendbus = inMixer.postSendBus;	// else, use existing bus
			});

			inMixer.postSendXferSynth.isNil.if({
					// place the xfer synth after the fader
					// take from sendbus, send to mixerchannel out
				xferSynth = Synth.tail(inMixer.fadergroup, 
					"mixers/xfer" ++ inMixer.outChannels,
					[\busin, sendbus.index, \busout, inMixer.outbus.index]
				);
				inMixer.postSendXferSynth = xferSynth;	
			}, {
				xferSynth = inMixer.postSendXferSynth;   // else use existing synth
			});
		}, {
			sendbus = nil;
			xferSynth = nil;
		});
		
			// check numchannels
			// outChannels here b/c this is postfader, using output channels
		(inMixer.outChannels != outbus.numChannels).if({
			("Warning: numChannel mismatch creating send for " ++ inMixer.name
				++ " -- " ++ inMixer.outChannels ++ " --> " ++ outbus.numChannels).postln;
		});
		
			// reroute mixerchannel output to transfer bus
		{
			inMixer.postSendReady.not.if({ 
				inMixer.synth.set(\busout, sendbus.index)
			});

				// this is the send itself, not merely a transfer			// takes from sendbus, sends to the send destination
			sendSynth = Synth.after(inMixer.synth, // place after mixer synth
				"mixers/Send" ++ inMixer.outChannels,
				[\busin, inMixer.postSendReady.if(
					{ inMixer.inbus.index },
					{ sendbus.index }),
				 \busout, outbus.index,
				 \level, level]
			);
			nil
		}.defer(inMixer.server.latency + 0.1);	// need time for xferSynth to be created
		
		guiIndex = inMixer.postSends.size;
		inMixer.postSends = inMixer.postSends.add(this);
			// fix gui display
		inMixer.mcgui.notNil.if({ inMixer.mcgui.refresh });
		
		this.outbus_(outbus, false, target, moveAction);
		
		^this
	}
	
	free {
		arg updateGUI = true;
		
//		updateGUI.if({
//		});

		this.stopLevelAuto;	// cancel automation

		inMixer.postSends.remove(this);	// pull from channel

		sendSynth.free;	// stop sending
		levelbus.isNil.not.if({ BusDict.free(levelbus); });   // free the kr bus

			// if we hit zero, free the xfer synth
		((inMixer.postSends.size == 0) && inMixer.postSendReady.not).if({
			inMixer.synth.set(\busout, inMixer.outbus.index);
			xferSynth.free;
			BusDict.free(sendbus);
		});

			// fix gui display
		(inMixer.mcgui.notNil && updateGUI).if({ inMixer.mcgui.board.refresh });

	}

	
}
