
MixerSkin {
	// holds display parameters for a MixingBoard
	// currently allows changing sizes of controls
	// will allow custom colors etc.
	// H. James Harkins - jamshark70@yahoo.com

	classvar <>screenHeight,	// if default height is wrong, you can change it for
			<>screenWidth;	// all skins
	
	var	<channelSize,	// dimensions of 1 mixer
		<gap,			// window margins
		<maxSize,		// of whole window
		<maxAcross,		// maximum channels across screen

		<nameBounds, <levelView,	// bounds for objects
		<levelSlider, <panSlider,	// assuming channel origin is Pt(0,0)
		<destMenu,
		<firstPreSendLevel, <firstPostSendLevel,
		<firstPreSendDest, <firstPostSendDest,
		<preSendIncr, <postSendIncr,	// amount to move for each pre/post send

		<preSendsShow = 1,		// how many pre-fader sends to show
		<postSendsShow = 1,		// post-fader sends
			// you can have more sends, but they won't display all at once
		
		<textSize,			// size of text objects
		<levelSliderSize,		// size of level slider
		<panSliderSize,		// and pan slider
		<sendDestSize,		// send: menu for destination
		<sendLevelSize,		// level slider

		<>autoFixBounds = true,		// automatically recalc all view bounds
								// upon a setter
		<>autoRefresh = true,		// if true, each setter refreshes all boards

		<layout = \vertical,		// horizontal or vertical?

		<textFont,
		
		<>levelSliderBackground,	// colorize sliders
		<>panSliderBackground,
		<>textColor,
		<>textBackGround,
		
		<color1,
		<color2,
		
		<viewGap = 6;		// space between views
	
	*initClass {
		Class.initClassTree(PageLayout);
		screenHeight = PageLayout.screenHeight;	// crib height from felix!! Thanks!
		screenWidth = PageLayout.screenWidth;
	}
	
	*new { arg layout = \vertical;	// set to \horizontal for a different look
		^super.new.init(layout);	// design your skin using setters
					// args may be implemented later
	}
	
	init { arg layout1;
		
		layout = layout1 ?? \vertical;
		
		color1 = Color.new255(84, 31, 77);
		color2 = Color.new255(230, 109, 225);
		textFont = Font("Helvetica", 9);		// small
		
		^this.fixViewSizes.fixViewBounds.fixBackgrounds;
	}
	
	fixViewSizes {
			// dimensions are specified in terms of points.
			// the point is the lower-right of a rect. starting at (0,0)
		(layout == \horizontal).if({
			textSize = Point(50, 20);
			gap = Point(10, 10);
			maxSize = Point(screenWidth, screenHeight);
			viewGap = 10;
			levelSliderSize = Point(100, textSize.y);
			panSliderSize = Point(40, textSize.y);
			sendDestSize = textSize;
			sendLevelSize = Point(textSize.x, 5);

			this.fixChanSize;
		}, {
			gap = Point(10, 15);
			maxSize = Point(1000, 800);
			textSize = Point(50, 20);
			viewGap = 10;
			levelSliderSize = Point(30, 200);
			panSliderSize = Point(textSize.x, 15);
			sendDestSize = textSize;
			sendLevelSize = Point(textSize.x, 5);	// full width, but narrow

			this.fixChanSize;
		});
	}
	
	fixChanSize {
			// calculate size needed for all views in a channel
			// with given sizes for each view
			// views needed: presends, pan, level, levelnumeric, postsends, name, dest
			// gaps needed:  preSShow, 1,   1,     1,            postSShow, 1,    0
		
		(layout == \horizontal).if({
			channelSize = Point.new(
					// gaps
				((preSendsShow + postSendsShow + 4) * viewGap) +
					// views
				((sendDestSize.x + (viewGap/3)) * preSendsShow) +
				panSliderSize.x + levelSliderSize.x + (textSize.x * 2) +
				((sendDestSize.x + (viewGap/3)) * postSendsShow) +
				sendDestSize.x,
					// y
				textSize.y + viewGap
			);
		}, {
			channelSize = Point.new(textSize.x,
					// get gaps
				((preSendsShow + postSendsShow + 4) * viewGap) +
					// views
				((sendDestSize.y + (viewGap/3) + sendLevelSize.y) * preSendsShow) +
				panSliderSize.y + levelSliderSize.y + (textSize.y * 2) +
				((sendDestSize.y + (viewGap/3) + sendLevelSize.y) * postSendsShow) +
				sendDestSize.y
			);
		});
	}	
	
	fixViewBounds {
			// calculate Rects for gui views
		(layout == \horizontal).if({
			nameBounds = Rect(0, viewGap, textSize.x, textSize.y);
			preSendIncr = Point(sendDestSize.x + (viewGap), 0);
sendDestSize.dump;
preSendIncr.dump;
			postSendIncr = preSendIncr;
			firstPreSendDest = Rect(nameBounds.right + viewGap, viewGap,
						sendDestSize.x, sendDestSize.y);
			firstPreSendLevel = Rect(nameBounds.right + viewGap, 0,
						sendLevelSize.x, sendLevelSize.y);
			panSlider = Rect(firstPreSendDest.left + (preSendIncr.x * preSendsShow), 
						viewGap, panSliderSize.x, panSliderSize.y);
			levelSlider = Rect(panSlider.right + viewGap, viewGap,
						levelSliderSize.x, levelSliderSize.y);
			levelView = Rect(levelSlider.right + viewGap, viewGap,
						textSize.x, textSize.y);
			firstPostSendDest = Rect(levelView.right + viewGap, viewGap,
						sendDestSize.x, sendDestSize.y);
			firstPostSendLevel = Rect(levelView.right + viewGap, 0,
						sendLevelSize.x, sendLevelSize.y);
			destMenu = Rect(firstPostSendDest.left + (postSendIncr.x * postSendsShow), viewGap,
						sendDestSize.x, sendDestSize.y);
		}, {
			preSendIncr = Point(0, sendDestSize.y + sendLevelSize.y + (viewGap*4/3));
			postSendIncr = preSendIncr;
			destMenu = Rect(0, channelSize.y - textSize.y - 15, 
						sendDestSize.x, sendDestSize.y);
			nameBounds = Rect(0, destMenu.top - sendDestSize.y - viewGap, 
						sendDestSize.x, sendDestSize.y);
			firstPostSendLevel = Rect(0, nameBounds.top - (postSendIncr.y * postSendsShow),
						sendLevelSize.x, sendLevelSize.y);
			firstPostSendDest = Rect(0, firstPostSendLevel.top + (viewGap/3) + sendLevelSize.y,
						sendDestSize.x, sendDestSize.y);
			levelView = Rect(0, firstPostSendLevel.top - textSize.y,
						textSize.x, textSize.y);
			levelSlider = Rect((channelSize.x - levelSliderSize.x) / 2, 
						levelView.top - viewGap - levelSliderSize.y,
						levelSliderSize.x, levelSliderSize.y);
			panSlider = Rect((channelSize.x - panSliderSize.x) / 2,
						levelSlider.top - viewGap - panSliderSize.y,
						panSliderSize.x, panSliderSize.y);
			firstPreSendLevel = Rect(0, 0, sendLevelSize.x, sendLevelSize.y);
			firstPreSendDest = Rect(0, viewGap/3 + sendLevelSize.y,
						sendDestSize.x, sendDestSize.y);
		});
		^this
	}
	
	fixBackgrounds {
		panSliderBackground = HiliteGradient(color2, color1, \h, 50, 0.5);				// note, pan is always horiz.
		levelSliderBackground = (layout == \horizontal).if(
			{ Gradient(color1, color2, \h, 50) },
			{ Gradient(color2, color1, \v, 50) }
		);
		
		^this
	}
	
	facelift { arg newskin;
"calling facelift".postln;
		MixingBoard.facelift(this, newskin);
	}
	
/////////////////////////// SETTERS ///////////////////////////
	
		// these setters were generated using the metacode commented-out
		// at the end of this file
		
		// (refresh ?? autoRefresh) allows the user to override the default refresh setting

	channelSize_ { arg newvalue, refresh;
//		var old;
//		old = this.copy;
		channelSize = newvalue;
		(autoFixBounds || (refresh ?? false)).if({ this.fixChanSize.fixViewBounds });
		(refresh ?? autoRefresh).if({ MixingBoard.facelift });
		^this
	}
	
	gap_ { arg newvalue, refresh;
//		var old;
//		old = this.copy;
		gap = newvalue;
		(autoFixBounds || (refresh ?? false)).if({ this.fixChanSize.fixViewBounds });
		(refresh ?? autoRefresh).if({ MixingBoard.facelift });
		^this
	}
	
	maxSize_ { arg newvalue, refresh;
//		var old;
//		old = this.copy;
		maxSize = newvalue;
		(autoFixBounds || (refresh ?? false)).if({ this.fixChanSize.fixViewBounds });
		(refresh ?? autoRefresh).if({ MixingBoard.facelift });
		^this
	}
	
	preSendsShow_ { arg newvalue, refresh;
//		var old;
//		old = this.copy;
		(newvalue < preSendsShow).if({ 
			"Warning: reducing the number of visible sends will mess up your mixingboard."
				.postln;
		});
		preSendsShow = newvalue;
		(autoFixBounds || (refresh ?? false)).if({ this.fixChanSize.fixViewBounds });
		(refresh ?? autoRefresh).if({ MixingBoard.facelift });
		^this
	}
	
	postSendsShow_ { arg newvalue, refresh;
//		var old;
//		old = this.copy;
		(newvalue < postSendsShow).if({ 
			"Warning: reducing the number of visible sends will mess up your mixingboard."
				.postln;
		});
		postSendsShow = newvalue;
		(autoFixBounds || (refresh ?? false)).if({ this.fixChanSize.fixViewBounds });
		(refresh ?? autoRefresh).if({ MixingBoard.facelift });
		^this
	}
	
	textSize_ { arg newvalue, refresh;
//		var old;
//		old = this.copy;
		textSize = newvalue;
		(autoFixBounds || (refresh ?? false)).if({ this.fixChanSize.fixViewBounds });
		(refresh ?? autoRefresh).if({ MixingBoard.facelift });
		^this
	}
	
	levelSliderSize_ { arg newvalue, refresh;
//		var old;
//		old = this.copy;
		levelSliderSize = newvalue;
		(autoFixBounds || (refresh ?? false)).if({ this.fixChanSize.fixViewBounds });
		(refresh ?? autoRefresh).if({ MixingBoard.facelift });
		^this
	}
	
	panSliderSize_ { arg newvalue, refresh;
//		var old;
//		old = this.copy;
		panSliderSize = newvalue;
		(autoFixBounds || (refresh ?? false)).if({ this.fixChanSize.fixViewBounds });
		(refresh ?? autoRefresh).if({ MixingBoard.facelift });
		^this
	}
	
	sendDestSize_ { arg newvalue, refresh;
//		var old;
//		old = this.copy;
		sendDestSize = newvalue;
		(autoFixBounds || (refresh ?? false)).if({ this.fixChanSize.fixViewBounds });
		(refresh ?? autoRefresh).if({ MixingBoard.facelift });
		^this
	}
	
	sendLevelSize_ { arg newvalue, refresh;
//		var old;
//		old = this.copy;
		sendLevelSize = newvalue;
		(autoFixBounds || (refresh ?? false)).if({ this.fixChanSize.fixViewBounds });
		(refresh ?? autoRefresh).if({ MixingBoard.facelift });
		^this
	}
	
	layout_ { arg newvalue, refresh;
//		var old;
//		old = this.copy;
		layout = newvalue;
		(autoFixBounds || (refresh ?? false)).if({
			this.fixViewSizes.fixChanSize.fixViewBounds
		});
		(refresh ?? autoRefresh).if({ MixingBoard.facelift });
		^this
	}
	
	viewGap_ { arg newvalue, refresh;
//		var old;
//		old = this.copy;
		viewGap = newvalue;
		(autoFixBounds || (refresh ?? false)).if({ this.fixChanSize.fixViewBounds });
		(refresh ?? autoRefresh).if({ MixingBoard.facelift });
		^this
	}
	
		// these setters are different because they don't change any view sizes
	color1_ { arg color, refresh;
		color1 = color;
		this.fixBackgrounds;
		(refresh ?? autoRefresh).if({ MixingBoard.facelift });
		^this
	}
	
	color2_ { arg color, refresh;
		color2 = color;
		this.fixBackgrounds;
		(refresh ?? autoRefresh).if({ MixingBoard.facelift });
		^this
	}

	textFont_ { arg font, refresh;
		textFont = font;
		(autoFixBounds || (refresh ?? false)).if({ this.fixChanSize.fixViewBounds });
		(refresh ?? autoRefresh).if({ MixingBoard.facelift });
		^this
	}
	
}

	// metacode to generate skin setters

//v = #[\channelSize, \gap, \maxSize, \preSendsShow, \postSendsShow, \textSize, \levelSliderSize, \panSliderSize, \sendDestSize, \sendLevelSize, \layout, \viewGap];
//
//// \nameBounds, \levelView, \levelSlider, \panSlider, \firstPreSendLevel, \firstPostSendLevel, \firstPreSendDest, \firstPostSendDest, \preSendIncr, \postSendIncr, 
//
//v.do({
//	arg a;
//	(a.asString ++ "_ { arg newvalue, refresh;").postln;
//	"\tvar old;".postln;
//	"\told = this.copy;".postln;
//	("\t" ++ a.asString ++ " = newvalue;").postln;
//	"\t(autoFixBounds || (refresh ?? false)).if({ this.fixChanSize.fixViewBounds });".postln;
//	"\t(refresh ?? autoRefresh).if({ old.facelift(this) });".postln;
//	"\t^this".postln;
//	"}\n".postln;
//});

